using System;
using System.Net;
using System.Web.Services.Protocols;
using System.Xml;
using Mintfix.Client.NotificationService;

namespace Mintfix.Client
{

    public class NotificationServiceWrapper : NotificationService.NotificationService
	{
        
		private AuthResult _ConnectionStatus = AuthResult.NoError;
		private string _Error = string.Empty;
        private string _ErrorMessage = string.Empty;
        private static DateTime? _LastTry;

        private static DateTime LastTry
        {
            get
            {
                if (_LastTry == null)
                {
                    _LastTry = DateTime.Now.AddMinutes(-SettingsManager.GetPeriod());

                }
                return _LastTry.Value;
            }
            set
            {
                _LastTry = value;
            }
        }

		public string LastError
		{
			get {return _Error;}
		}

        public string LastErrorMessage
        {
            get { return _ErrorMessage; }
        }
		
		public AuthResult ConnectionStatus
		{
			get {return _ConnectionStatus;}
		}
		
		public string AuthInfo= string.Empty;

    
        public NotificationServiceWrapper()
        {
            UseDefaultCredentials = true;
        }
     

        public DataTransferObject[] GetNotifications()
		{
			_Error = String.Empty;
			_ConnectionStatus = AuthResult.NoError;
			try
			{
                Url = SettingsManager.GetServiceUrl();
			    var requestDate = DateTime.Now;
                DataTransferObject[] data = base.GetNotifications(String.Empty, LastTry, requestDate);
                LastTry = requestDate;              
				return data;
			}
			catch(Exception exc)
			{
			    HandleException(exc);
			    return null;
			}

		}

		public  bool CanConnect
		{
			get 
			{
				_Error = String.Empty;
				_ConnectionStatus = AuthResult.NoError;
				try
				{
                    Url = SettingsManager.GetServiceUrl();
					return this.Ping();	
				}
                catch (Exception exc)
                {
                    HandleException(exc);
                    return false;
                }
			}
		}

        public void HandleException(Exception exc)
        {
            _Error = exc.ToString();
            _ErrorMessage = GetFriendlyMessage(exc);

            if( exc is  SoapException)
			{
                _ConnectionStatus = AuthResult.ServiceError;
				if( exc.Message.IndexOf("Access Denied", StringComparison.Ordinal) >= 0 )
				{
					_ConnectionStatus = AuthResult.AuthError;

                    _Error = String.Empty;
                    _ErrorMessage = String.Empty;
				}
			}
			else if(exc is WebException)
            {
                var we = (WebException) exc;
                _ConnectionStatus = AuthResult.ConnectionFailed;
				if( we.Status == WebExceptionStatus.UnknownError )
				{
					_ConnectionStatus = AuthResult.UnknownError;
				}
			}
			else
			{
				_ConnectionStatus = AuthResult.UnknownError;
			}

        }

        /// <summary>
        ///    
        /// </summary>
        /// <param name="exc"></param>
        /// <returns></returns>
        public static string GetFriendlyMessage(Exception exc)
        {
            string addmessage = String.Empty;
            SoapException e = exc as SoapException;
            if (e != null)
            {
                if (e.Detail != null)
                {
                    XmlNode nodeDescr = e.Detail.SelectSingleNode("/error/description");
                    addmessage = ((nodeDescr != null) ? nodeDescr.InnerText : String.Empty);
                }
            }

            return exc.Message + ":" + addmessage;
        }
	}
}
