using System;
using System.Diagnostics;
using System.Threading;
using Mintfix.Client.NotificationService;
using Mintfix.Client.UI;

namespace Mintfix.Client.Tasks
{
	/// <summary>
	/// Summary description for TaskThreadManager.
	/// </summary>
	public sealed class TaskThreadManager
	{
		private bool _IsRunning = false;
		private readonly MainForm _ParentForm = null;
		private Thread _TaskThread = null;
		private readonly AutoResetEvent _EventCheckData = new AutoResetEvent(false);
		private readonly AutoResetEvent _EventLeaveThread = new AutoResetEvent(false);

		public bool IsRunning
		{
			get { return _IsRunning; }
		}

		public void EndTask()
		{
			if( IsRunning )
			{
				_EventLeaveThread.Set();
				_TaskThread.Join();
			}
		}

		public void CheckNow()
		{
            if (IsRunning)
            {
                _EventCheckData.Set();
            }
            else
            {
                Run();
            }
		}


		public TaskThreadManager(MainForm form)
		{
		    if (form == null) 
                throw new ArgumentNullException("form");

			this._ParentForm = form;
		}

	    public void Run()
		{
			_TaskThread = new Thread(MainTask) {IsBackground = true, Name = "TaskThread"};
		    _TaskThread.Start();
		}

		private void MainTask()
		{
			if( IsRunning ) return;
			
			_IsRunning = true;

			var service = new NotificationServiceWrapper();
			try
			{
			    while (true)
				{
					DataTransferObject[] messages = service.GetNotifications();

                    if ((messages != null)&&(messages.Length >0))
					{
						_ParentForm.ShowForm(messages);
					}
					else if (service.ConnectionStatus != AuthResult.NoError)
					{
						Logger.WriteLog("WorkService",Helper.GetDescription(service.ConnectionStatus), service.LastError );
					}

                    _ParentForm.ShowConnectionState(service.ConnectionStatus, service.LastErrorMessage);

					var xWaits = new WaitHandle[] {_EventCheckData, _EventLeaveThread};
						
					int xIndex = WaitHandle.WaitAny(xWaits, SettingsManager.GetPeriod()*60*1000, true);
					
                    //check now handled
                    if (xIndex == 0)
					{
						continue;
					}

                    //exit handled
				    if (xIndex == 1)
				    {
				        _IsRunning = false;
				        return;
				    }
				}
			}
			catch
			{
				_IsRunning = false;
			}
		}
	}
}
