using System;
using System.ComponentModel;
using System.Globalization;
using System.Windows.Forms;
using Mintfix.Client.Properties;

namespace Mintfix.Client.UI
{
	/// <summary>
	/// Summary description for Settings.
	/// </summary>
	

	public class Settings : Form
	{
		public AuthResult AuthStatus;
        public EditSettingsResult EditStatus;
		private Button btnAuth;
		private Label lblError;

        private CheckBox cbAutoLaunch;
        private Label label2;
        private NumericUpDown numPeriod;
        private Label label1;
        private Label label3;
        private TextBox txtKey;
        private Label label4;
        private TextBox txtServer;
        private Panel panel1;
        private Label label5;
    
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private Container components = null;
		

		public Settings()
		{
			InitializeComponent();

            cbAutoLaunch.Checked = SettingsManager.GetAutorun();
			numPeriod.Value = SettingsManager.GetPeriod() ;
		    numPeriod.Text = numPeriod.Value.ToString(CultureInfo.InvariantCulture);
		    txtServer.Text = SettingsManager.GetServiceUrl();

			EditStatus = EditSettingsResult.Close;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		

		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		
		private void InitializeComponent()
		{
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(Settings));
            this.btnAuth = new System.Windows.Forms.Button();
            this.lblError = new System.Windows.Forms.Label();
            this.cbAutoLaunch = new System.Windows.Forms.CheckBox();
            this.label2 = new System.Windows.Forms.Label();
            this.numPeriod = new System.Windows.Forms.NumericUpDown();
            this.label1 = new System.Windows.Forms.Label();
            this.label3 = new System.Windows.Forms.Label();
            this.txtKey = new System.Windows.Forms.TextBox();
            this.label4 = new System.Windows.Forms.Label();
            this.txtServer = new System.Windows.Forms.TextBox();
            this.panel1 = new System.Windows.Forms.Panel();
            this.label5 = new System.Windows.Forms.Label();
            ((System.ComponentModel.ISupportInitialize)(this.numPeriod)).BeginInit();
            this.panel1.SuspendLayout();
            this.SuspendLayout();
            // 
            // btnAuth
            // 
            resources.ApplyResources(this.btnAuth, "btnAuth");
            this.btnAuth.Name = "btnAuth";
            this.btnAuth.Click += new System.EventHandler(this.btnAuth_Click);
            // 
            // lblError
            // 
            resources.ApplyResources(this.lblError, "lblError");
            this.lblError.ForeColor = System.Drawing.Color.DarkRed;
            this.lblError.Name = "lblError";
            // 
            // cbAutoLaunch
            // 
            resources.ApplyResources(this.cbAutoLaunch, "cbAutoLaunch");
            this.cbAutoLaunch.Name = "cbAutoLaunch";
            // 
            // label2
            // 
            resources.ApplyResources(this.label2, "label2");
            this.label2.Name = "label2";
            // 
            // numPeriod
            // 
            resources.ApplyResources(this.numPeriod, "numPeriod");
            this.numPeriod.Maximum = new decimal(new int[] {
            20,
            0,
            0,
            0});
            this.numPeriod.Minimum = new decimal(new int[] {
            1,
            0,
            0,
            0});
            this.numPeriod.Name = "numPeriod";
            this.numPeriod.Value = new decimal(new int[] {
            15,
            0,
            0,
            0});
            // 
            // label1
            // 
            resources.ApplyResources(this.label1, "label1");
            this.label1.Name = "label1";
            // 
            // label3
            // 
            resources.ApplyResources(this.label3, "label3");
            this.label3.Name = "label3";
            // 
            // txtKey
            // 
            resources.ApplyResources(this.txtKey, "txtKey");
            this.txtKey.BackColor = System.Drawing.SystemColors.ControlLight;
            this.txtKey.Name = "txtKey";
            // 
            // label4
            // 
            resources.ApplyResources(this.label4, "label4");
            this.label4.Name = "label4";
            // 
            // txtServer
            // 
            resources.ApplyResources(this.txtServer, "txtServer");
            this.txtServer.Name = "txtServer";
            // 
            // panel1
            // 
            resources.ApplyResources(this.panel1, "panel1");
            this.panel1.BackColor = System.Drawing.Color.White;
            this.panel1.Controls.Add(this.label5);
            this.panel1.Name = "panel1";
            // 
            // label5
            // 
            resources.ApplyResources(this.label5, "label5");
            this.label5.Name = "label5";
            // 
            // Settings
            // 
            resources.ApplyResources(this, "$this");
            this.Controls.Add(this.panel1);
            this.Controls.Add(this.txtServer);
            this.Controls.Add(this.label4);
            this.Controls.Add(this.txtKey);
            this.Controls.Add(this.label3);
            this.Controls.Add(this.cbAutoLaunch);
            this.Controls.Add(this.label2);
            this.Controls.Add(this.numPeriod);
            this.Controls.Add(this.label1);
            this.Controls.Add(this.lblError);
            this.Controls.Add(this.btnAuth);
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
            this.MaximizeBox = false;
            this.MinimizeBox = false;
            this.Name = "Settings";
            this.ShowInTaskbar = false;
            this.Load += new System.EventHandler(this.Settings_Load);
            ((System.ComponentModel.ISupportInitialize)(this.numPeriod)).EndInit();
            this.panel1.ResumeLayout(false);
            this.panel1.PerformLayout();
            this.ResumeLayout(false);
            this.PerformLayout();

		}


	    private void btnAuth_Click(object sender, EventArgs e)
		{
            if(String.IsNullOrEmpty(txtServer.Text))
            {
                if(DialogResult.No == MessageBox.Show(Resources.ServiceUrlRequired, Resources.SettingsError, MessageBoxButtons.YesNo, MessageBoxIcon.Warning))
                {
                    DialogResult = DialogResult.No;
                    Close(); 
                }
            }
            else
            {
                lblError.Text = String.Empty;
                SettingsManager.SetPeriod((int)numPeriod.Value);
                SettingsManager.SetServiceUrl(txtServer.Text);
                SettingsManager.SetAutorun(cbAutoLaunch.Checked);

                SettingsManager.Save();
                Close();
            }

		}

        private void Settings_Load(object sender, EventArgs e)
        {

        }
	
	}
}
