﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Data.SqlClient;
using System.Collections.Specialized;
using System.Net.Mail;
using System.IO;
using System.Reflection;
using Mintfix.Web.Interfaces;

namespace Mintfix.Web.Crm
{
    public class DataHandler : IDataTransferObjectFactory
    {

        private NameValueCollection _Configuration = null;

        #region IDataTransferObjectFactory Members

        public void Configure(NameValueCollection configuration)
        {
            if (configuration == null)
            {
                throw new ArgumentNullException("configuration");
            }
            _Configuration = configuration;
        }
        


        public DataTransferObject[] GetObjects(string username, DateTime dateStart, DateTime dateEnd)
        {
            //int sendByEmail = Convert.ToInt32(_Configuration["Email"]);

            var result = new List<DataTransferObject>();
            var r1 = GetEvents(username, dateStart, dateEnd);
            var r2 = GetTasks(username, dateStart, dateEnd);

            result.AddRange(r1);
            result.AddRange(r2);

            return result.ToArray();
        }

        #endregion

        private void SendByEmail(string username, IEnumerable<DataTransferObject> result)
        {
            string email = String.Empty;
            string queryUser = @"	SELECT internalemailaddress FROM SystemUser 
				                        WHERE domainname = @owner";

            using (SqlConnection connection = new SqlConnection(_Configuration["Crm"]))
            {
                connection.Open();
                using (SqlCommand cmd = new SqlCommand(queryUser, connection))
                {
                    cmd.Parameters.AddWithValue("@owner", username);

                    email = cmd.ExecuteScalar() as string ?? String.Empty;
                }
            }

            if (!String.IsNullOrEmpty(email))
            {
                SmtpClient smtp = new SmtpClient();
                string headerTemplate = GetTemplate("Promintel.Notifications.header.htm");
                string body = String.Empty;
                ;
                string bodyTemplate = GetTemplate("Promintel.Notifications.template.htm");
                ;
                foreach (DataTransferObject dto in result)
                {
                    string localTemplate = bodyTemplate.Replace("%link%", dto.Href);
                    localTemplate = localTemplate.Replace("%subject%", dto.Subject);
                    localTemplate = localTemplate.Replace("%date%", dto.PublishDate.ToShortDateString());
                    localTemplate = localTemplate.Replace("%time%", dto.PublishDate.ToShortTimeString());
                    localTemplate = localTemplate.Replace("%location%", dto.Description);

                    body += localTemplate;
                }

                MailMessage msg = new MailMessage();
                msg.IsBodyHtml = true;
                msg.To.Add(email);
                msg.Subject = "CRM\\Напоминание";
                msg.Body = headerTemplate.Replace("%template%", body);
                smtp.Send(msg);
            }
        }

        

        private IEnumerable<DataTransferObject> GetEvents(string username, DateTime dateStart, DateTime dateEnd)
        {
            int minutesBefore = Convert.ToInt32(_Configuration["MinutesBefore"]);
            dateStart = dateStart.AddMinutes(minutesBefore);
            dateEnd = dateEnd.AddMinutes(minutesBefore);

            var result = new List<DataTransferObject>();


            //2. SQL запрос
            string query = @"select activityid, subject, location, scheduledstart, owneridname from FilteredAppointment
	                        where
		                        scheduledstart IS NOT NULL AND
		                        scheduledstart >= @dateStart AND
		                        scheduledstart < @dateEnd AND
		                        (
			                        @owner IN 
				                        (
					                        SELECT u.domainname FROM ActivityParty ap
						                        INNER JOIN FilteredSystemUser u ON u.systemuserid = ap.partyid AND ap.partyobjecttypecode = 8
						                        WHERE ap.activityid = FilteredAppointment.activityid
				                        )
		                        )";

            //3. результирующий массив
            using (SqlConnection connection = new SqlConnection(_Configuration["Crm"]))
            {
                connection.Open();
                using (SqlCommand cmd = new SqlCommand(query, connection))
                {
                    cmd.Parameters.AddWithValue("@owner", username);
                    cmd.Parameters.AddWithValue("@dateStart", dateStart.AddHours(-4));
                    cmd.Parameters.AddWithValue("@dateEnd", dateEnd.AddHours(-4));

                    SqlDataReader reader = cmd.ExecuteReader(System.Data.CommandBehavior.CloseConnection);
                    if (reader.HasRows)
                    {
                        while (reader.Read())
                        {
                            DataTransferObject obj = new DataTransferObject();
                            obj.Type = DataType.Appointment;
                            obj.Id = reader["activityid"].ToString();
                            obj.Subject = reader["subject"].ToString();
                            obj.OwnerName = reader["owneridname"].ToString();
                            obj.PublishDate = DateTime.Parse(reader["scheduledstart"].ToString()).AddHours(4);
                            obj.Href = String.Format(_Configuration["UrlVisit"], obj.Id);
                            obj.Description = reader["location"].ToString();

                            result.Add(obj);
                        }
                    }
                }
            }

            return result.ToArray();
        }

        private IEnumerable<DataTransferObject> GetTasks(string username, DateTime dateStart, DateTime dateEnd)
        {
            int minutesBefore = Convert.ToInt32(_Configuration["MinutesBefore"]);
            dateStart = dateStart.AddMinutes(minutesBefore);
            dateEnd = dateEnd.AddMinutes(minutesBefore);

            var result = new List<DataTransferObject>();

            //1. Строка соединения
            string connectionString = _Configuration["Crm"];

            //2. SQL запрос
            string query = @"select activityid, subject, description, scheduledend, owneridname from FilteredTask
	                        where
		                        scheduledend IS NOT NULL AND
		                        scheduledend >= @dateStart AND
		                        scheduledend < @dateEnd AND
		                        (
			                        @owner IN 
				                        (
					                        SELECT u.domainname FROM ActivityParty ap
						                        INNER JOIN FilteredSystemUser u ON u.systemuserid = ap.partyid AND ap.partyobjecttypecode = 8
						                        WHERE ap.activityid = FilteredTask.activityid
				                        )
		                        )";

            //3. результирующий массив
            using (var connection = new SqlConnection(connectionString))
            {
                connection.Open();
                using (var cmd = new SqlCommand(query, connection))
                {
                    cmd.Parameters.AddWithValue("@owner", username);
                    cmd.Parameters.AddWithValue("@dateStart", dateStart);
                    cmd.Parameters.AddWithValue("@dateEnd", dateEnd.AddHours(-4));

                    SqlDataReader reader = cmd.ExecuteReader(System.Data.CommandBehavior.CloseConnection);
                    if (reader.HasRows)
                    {
                        while (reader.Read())
                        {
                            var obj = new DataTransferObject
                                          {
                                              Type = DataType.Task,
                                              Id = reader["activityid"].ToString(),
                                              Subject = reader["subject"].ToString(),
                                              OwnerName = reader["owneridname"].ToString(),
                                              PublishDate = DateTime.Parse(reader["scheduledend"].ToString())
                                          };
                            obj.Href = String.Format(_Configuration["UrlTask"], obj.Id);
                            obj.Description = reader["description"].ToString();

                            result.Add(obj);
                        }
                    }
                }
            }
            return result.ToArray();
        }

        public string GetTemplate(string templateName)
        {
            StringBuilder template = new StringBuilder();
            using (Stream stream = Assembly.GetExecutingAssembly().GetManifestResourceStream(templateName))
            {
                using (StreamReader reader = new StreamReader(stream))
                {
                    template.Append(reader.ReadToEnd());
                }
            }

            return template.ToString();
        }
    }
}
