﻿using System;
using System.Web.Services;
using System.Xml.Serialization;
using System.Configuration;
using System.Collections.Specialized;
using Mintfix.Web.Interfaces;

namespace Mintfix.Web.Services
{
    /// <summary>
    /// Notification service
    /// </summary>
    [WebService(Namespace = "http://mintfix.ru/")]
    [WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
    [System.ComponentModel.ToolboxItem(false)]
    [System.Web.Script.Services.ScriptService]
    public class NotificationService : System.Web.Services.WebService
    {

        [WebMethod( Description = "Pinging service")]
        public bool Ping()
        {
            return true;
        }

        [SoapInclude(typeof(DataTransferObject))]
        [SoapInclude(typeof(DataType))]
        [WebMethod( Description = "Return all notifiation for specified date range")]
        public DataTransferObject[] GetNotifications(string key, DateTime dateStart, DateTime dateEnd)
        {
            key = key.ToLower();


            try
            {
                NameValueCollection config = ConfigurationManager.AppSettings;

                if (!String.IsNullOrEmpty(key))
                {
                    config = (NameValueCollection)ConfigurationManager.GetSection(key);
                    if(config == null)
                    {
                        throw new ApplicationException("Configuration not found");
 
                    }
                }

                string typename = config["Type"];
                if (String.IsNullOrEmpty(typename))
                {
                    throw new ApplicationException("The Type configuration parameter cannot be empty.");
                }

                Type factoryType = Type.GetType(typename);

                if (factoryType == null)
                {
                    throw new ApplicationException("Type " + typename + " not found.");
                }

                IDataTransferObjectFactory factory = (IDataTransferObjectFactory)Activator.CreateInstance(factoryType);

                factory.Configure(config);

                var result =  factory.GetObjects(User.Identity.Name, dateStart, dateEnd);

                return result;
            }
            catch (Exception exc)
            {
                throw new ApplicationException("Request failed " + key + "; " + User.Identity.Name + ";" + dateStart + "; " + dateEnd + "; " + exc.Message);
            }
        }
    }
}
